# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/sec-downloader',
                'doc_host': 'https://Elijas.github.io',
                'git_url': 'https://github.com/Elijas/sec-downloader',
                'lib_path': 'sec_downloader'},
  'syms': { 'sec_downloader.core': {},
            'sec_downloader.download_storage': {},
            'sec_downloader.sec_edgar_downloader_fork': {},
            'sec_downloader.types': { 'sec_downloader.types.CompanyAndAccessionNumber': ( 'types.html#companyandaccessionnumber',
                                                                                          'sec_downloader/types.py'),
                                      'sec_downloader.types.CompanyAndAccessionNumber.from_string': ( 'types.html#companyandaccessionnumber.from_string',
                                                                                                      'sec_downloader/types.py'),
                                      'sec_downloader.types.FilingMetadata': ('types.html#filingmetadata', 'sec_downloader/types.py'),
                                      'sec_downloader.types.RequestedFilings': ('types.html#requestedfilings', 'sec_downloader/types.py'),
                                      'sec_downloader.types.RequestedFilings.from_string': ( 'types.html#requestedfilings.from_string',
                                                                                             'sec_downloader/types.py'),
                                      'sec_downloader.types.Ticker': ('types.html#ticker', 'sec_downloader/types.py')}}}
